//
// DataCache.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.net>
//
// Copyright (c) 2014 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Linq;
using System.Collections.Generic;
using Ntp.Data;

namespace Ntp.Analyzer.Data.Cache
{
    /// <summary>
    /// Base class for data caches.
    /// </summary>
    public abstract class DataCache<T> : DataMapper<T>
    {
        protected DataCache()
        {
            entries = new Dictionary<String, CacheEntry<T>>();
        }

        private readonly Dictionary<String, CacheEntry<T>> entries;
        private readonly object locker = new object();

        public T this[string name]
        {
            get { return GetImporter(name); }
        }

        protected abstract int ValidMinutes { get; }

        /// <summary>
        /// Read all data from cache in a sequential manner.
        /// </summary>
        /// <returns>The enumerator.</returns>
        public override IEnumerator<T> GetEnumerator()
        {
            lock (locker)
            {
                return entries.Values.Select(e => e.Entry).ToList().GetEnumerator();
            }
        }

        private T GetImporter(string name)
        {
            T entry;

            lock (locker)
            {
                if (entries.ContainsKey(name))
                {
                    if (entries[name].IsValid)
                        return entries[name].Entry;

                    entries.Remove(name);
                }

                entry = Create(name);
                entries.Add(name, new CacheEntry<T>(ValidMinutes, entry));
            }

            return entry;
        }

        protected abstract T Create(string name);
    }
}
