//
// DataCache.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Linq;
using System.Collections.Generic;

namespace Ntp.Analyzer.Data
{
	/// <summary>
	/// Base class for data caches.
	/// </summary>
	public abstract class DataCache<T> : DataMapper<T>
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="Ntp.Analyzer.Data.DataCache`1"/> class.
		/// </summary>
		protected DataCache ()
		{
			entries = new  Dictionary<String, CacheEntry<T>> ();			
		}

		private readonly Dictionary<String, CacheEntry<T>> entries;

		/// <summary>
		/// Read all data from cache in a sequential manner.
		/// </summary>
		/// <returns>The enumerator.</returns>
		public override IEnumerator<T> GetEnumerator ()
		{
			return entries.Values.Select (e => e.Entry).ToList ().GetEnumerator ();
		}

		/// <summary>
		/// Gets the <see cref="Ntp.Analyzer.Data.DataCache`1"/> with the specified name.
		/// </summary>
		/// <param name="name">Name.</param>
		public T this [string name] {
			get { return GetImporter (name); }
		}

		private T GetImporter (string name)
		{
			if (entries.ContainsKey (name)) {
				if (entries [name].IsValid)
					return entries [name].Entry;
				else
					entries.Remove (name);
			}
		
			T entry = Create (name);
			entries.Add (name, new CacheEntry<T> (ValidMinutes, entry));
			
			return entry;	
		}

		public override void CleanUp ()
		{
			// Nothing to clean up
		}

		protected abstract int ValidMinutes { get; }

		protected abstract T Create (string name);
	}
}
