//
// DatabaseMapper.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Data;
using System.Linq;
using System.Collections.Generic;
using MySql.Data;
using MySql.Data.MySqlClient;
using Ntp.Analyzer.Objects;

namespace Ntp.Analyzer.Data.Mapper
{
	/// <summary>
	/// Base class for ORM mappers. Can be used for mapping object stored in databases.
	/// </summary>
	/// <remarks>Based on MySql</remarks>
	public abstract class DatabaseMapper<T> : DataMapper<T>
		where T: PersistentObject
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="Ntp.Analyzer.Data.Mapper.DatabaseMapper`1"/> class.
		/// </summary>
		protected DatabaseMapper ()
		{
		}

		private readonly string connectionString = DataConnection.String;
		private MySqlConnection connection = null;
		private MySqlCommand command;
		private readonly Dictionary<int, T> items = new  Dictionary<int, T> ();
		private bool hasContent = false;

		protected abstract bool UseCache { get; }

		protected bool HasContent {
			get { return hasContent; }
		}

		/// <summary>
		/// Gets the <see cref="Ntp.Analyzer.Data.Mapper.DatabaseMapper`1"/> with the specified id.
		/// </summary>
		/// <param name="id">Identifier.</param>
		public T this [int id] { 
			get { 
				if (!UseCache)
					throw new NotSupportedException ("Mapper does not implement caching.");
			
				if (!HasContent)
					ReadContent ();
		
				if (items.ContainsKey (id)) {
					return items [id];
				} else {
					return FetchExternal (id);
				}
			}
		}

		protected virtual T FetchExternal (int id)
		{
			return null;
		}

		private void ReadContent ()
		{
			// Force fetch from db
			foreach (T item in this)
				;
		}

		protected void AddItem (T item)
		{
			items.Add (item.Id, item);
			hasContent = true;
		}

		protected IEnumerable<T> Content {
			get { return items.Values.Cast<T> (); }
		}

		protected MySqlCommand Command
		{ get { return command; } }

		protected void Open ()
		{
			if (connection == null)
				connection = new MySqlConnection (connectionString);
				
			if (connection.State == ConnectionState.Closed)
				connection.Open ();
						
			command = new MySqlCommand ();
			command.Connection = connection;	
		}

		public void Close ()
		{
			if (connection != null)
				connection.Close ();
				
			connection = null;
		}

		public override void CleanUp ()
		{
			Close ();
		}

		public void Save (T item)
		{
			Open ();
			
			try {
				if (item.NewObject) {
					Insert (item);
					
					if (UseCache)
						AddItem (item);
				} else
					Update (item);
			} catch (MySqlException ex) {
				Console.WriteLine ("Error: {0}", ex.ToString ());

			} finally {
				if (connection != null) {
					connection.Close ();
				}
			}
		}

		protected abstract void Insert (T item);

		protected abstract void Update (T item);
	}
}
