//
// HostReadingDatabaseMapper.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Data;
using MySql.Data;
using MySql.Data.MySqlClient;
using Ntp.Analyzer.Objects;

namespace Ntp.Analyzer.Data.Mapper
{
	/// <summary>
	/// ORM mapper for table hostReading.
	/// </summary>
	public sealed class HostReadingDatabaseMapper : DatabaseMapper<HostReading>
	{
		internal HostReadingDatabaseMapper (
			HostDatabaseMapper hostMapper,
			PeerDatabaseMapper peerMapper)
		{
			this.hostMapper = hostMapper;
			this.peerMapper = peerMapper;
		}

		protected override bool UseCache {
			get { return false; }
		}

		private readonly HostDatabaseMapper hostMapper;
		private readonly PeerDatabaseMapper peerMapper;

		protected override void Insert (HostReading item)
		{
			Open ();
			
			Command.CommandText = 
				"INSERT INTO hostReading (time, hostId, peerId, offset, jitter, frequency, stability) " +
				"VALUES ( @time, @hostId, @peerId, @offset, @jitter, @frequency, @stability);" +
				"SELECT LAST_INSERT_ID();";
			Command.Prepare ();
           
			Command.Parameters.AddWithValue ("@time", item.Time);
			Command.Parameters.AddWithValue ("@hostId", item.Host.Id);
			Command.Parameters.AddWithValue ("@peerId", item.Peer.Id);
			Command.Parameters.AddWithValue ("@offset", item.Offset);
			Command.Parameters.AddWithValue ("@jitter", item.Jitter);
			Command.Parameters.AddWithValue ("@frequency", item.Frequency);
			Command.Parameters.AddWithValue ("@stability", item.Stability);

			object idObject = Command.ExecuteScalar ();
			item.SetId (Convert.ToInt32 (idObject));
		}

		protected override void Update (HostReading item)
		{
			Open ();
			
			Command.CommandText = 
				"UPDATE hostReading SET " +
				"name = @name, " +
				"hostId = @hostId, " +
				"peerId = @peerId, " +
				"offset = @offset, " +
				"jitter = @jitter, " +
				"frequency = @frequency, " +
				"stability = @stability " +
				"WHERE id = @id;";
			Command.Prepare ();
           
			Command.Parameters.AddWithValue ("@time", item.Time);
			Command.Parameters.AddWithValue ("@hostId", item.Host.Id);
			Command.Parameters.AddWithValue ("@peerId", item.Peer.Id);
			Command.Parameters.AddWithValue ("@offset", item.Offset);
			Command.Parameters.AddWithValue ("@jitter", item.Jitter);
			Command.Parameters.AddWithValue ("@frequency", item.Frequency);
			Command.Parameters.AddWithValue ("@stability", item.Stability);
			Command.Parameters.AddWithValue ("@id", item.Id);
			Command.ExecuteNonQuery ();
		}

		/// <summary>
		/// Gets or sets the host 
		/// </summary>
		/// <value>The host.</value>
		public Host FilterHost { get; set; }

		/// <summary>
		/// Gets or sets the time to use when extracting data. Only readings with a timestamp
		/// later than FilterTime gets extracted.
		/// </summary>
		/// <remarks>Always set this property while using other filter properties</remarks>
		/// <value>The time.</value>
		public DateTime? FilterTime { get; set; }

		/// <summary>
		/// Read all data from table in a sequential manner.
		/// </summary>
		/// <returns>The enumerator.</returns>
		public override System.Collections.Generic.IEnumerator<HostReading> GetEnumerator ()
		{			
			Open ();
			
			string sql = "SELECT " +
				" id, time, hostId, peerId, offset, jitter, frequency, stability" +
				" FROM hostReading ";
			
			if (FilterTime != null)
				sql += " WHERE time > '" + FilterTime.Value.ToString ("yyyy-MM-dd HH:mm") + "'";	

			if (FilterHost != null)
				sql += " AND hostId = " + FilterHost.Id;
							
			Command.CommandText = sql;
			MySqlDataReader reader = Command.ExecuteReader ();

			while (reader.Read()) {
				int id = Convert.ToInt32 (reader ["id"]);
				DateTime time = Convert.ToDateTime (reader ["time"]);
				int hostId = Convert.ToInt32 (reader ["hostId"]);
				Host host = hostMapper [hostId];
				int peerId = Convert.ToInt32 (reader ["peerId"]);
				Peer peer = peerMapper [peerId];
				double offset = Convert.ToDouble (reader ["offset"]);
				double jitter = Convert.ToDouble (reader ["jitter"]);
				double frequency = Convert.ToDouble (reader ["frequency"]);
				double stability = Convert.ToDouble (reader ["stability"]);
				
				HostReading reading = new HostReading (
					id, time, host, peer, null,
					offset, jitter, frequency, stability);
				
				yield return reading;
			}
			
			Close ();
		}
	}
}
