//
// PeerDatabaseMapper.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Data;
using MySql.Data;
using MySql.Data.MySqlClient;
using Ntp.Analyzer.Objects;

namespace Ntp.Analyzer.Data.Mapper
{
	/// <summary>
	/// ORM mapper for table peer.
	/// </summary>
	public sealed class PeerDatabaseMapper : DatabaseMapper<Peer>
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="Ntp.Analyzer.Data.Mapper.PeerDatabaseMapper"/> class.
		/// </summary>
		/// <param name="timeServerMapper">Time server mapper.</param>
		internal PeerDatabaseMapper (TimeServerDatabaseMapper timeServerMapper)
		{
			this.timeServerMapper = timeServerMapper;
		}

		private readonly TimeServerDatabaseMapper timeServerMapper;

		protected override bool UseCache {
			get { return true; }
		}

		protected override void Insert (Peer item)
		{
			Open ();
			
			Command.CommandText = "INSERT INTO peer(name, ip, orgId) VALUES(@name, @ip, @orgId);" +
				"SELECT LAST_INSERT_ID();";
			Command.Prepare ();
           
			Command.Parameters.AddWithValue ("@name", item.Name);
			Command.Parameters.AddWithValue ("@ip", item.Ip);
			Command.Parameters.AddWithValue ("@orgId", item.OrgId);

			object idObject = Command.ExecuteScalar ();
			item.SetId (Convert.ToInt32 (idObject));
		}

		protected override void Update (Peer item)
		{
			Open ();
			
			Command.CommandText = "UPDATE peer SET name = @name, ip=@ip, orgId=@orgId WHERE id=@id";
			Command.Prepare ();
           
			Command.Parameters.AddWithValue ("@name", item.Name);
			Command.Parameters.AddWithValue ("@ip", item.Ip);
			Command.Parameters.AddWithValue ("@orgId", item.OrgId);
			Command.Parameters.AddWithValue ("@id", item.Id);
			Command.ExecuteNonQuery ();
		}

		/// <summary>
		/// Read all data from table in a sequential manner.
		/// </summary>
		/// <returns>The enumerator.</returns>
		public override System.Collections.Generic.IEnumerator<Peer> GetEnumerator ()
		{
			if (HasContent) {
				foreach (Peer item in Content)
					yield return item;
			
				yield break;
			}
							
			Open ();
			Command.CommandText = "SELECT id, name, ip, orgId FROM peer;";
			MySqlDataReader reader = Command.ExecuteReader ();

			while (reader.Read()) {
				int id = Convert.ToInt32 (reader ["id"]);
				string name = reader ["name"].ToString ();
				string ip = reader ["ip"].ToString ();
				int orgId = Convert.ToInt32 (reader ["orgId"]);
				
				TimeServer server = timeServerMapper [orgId];
				
				Peer peer = new Peer (id, name, ip, server);
				AddItem (peer);
				
				yield return peer;
			}
			
			Close ();
		}
	}
}
