//
// PeerReadingDatabaseMapper.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Data;
using System.Globalization;
using MySql.Data;
using MySql.Data.MySqlClient;
using Ntp.Analyzer.Objects;

namespace Ntp.Analyzer.Data.Mapper
{
	/// <summary>
	/// ORM mapper for table peerReading.
	/// </summary>
	public sealed class PeerReadingDatabaseMapper : DatabaseMapper<PeerReading>
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="Ntp.Analyzer.Data.Mapper.PeerReadingDatabaseMapper"/> class.
		/// </summary>
		/// <param name="hostMapper">Host mapper.</param>
		/// <param name="peerMapper">Peer mapper.</param>
		internal PeerReadingDatabaseMapper (
			HostDatabaseMapper hostMapper,
			PeerDatabaseMapper peerMapper)
		{
			this.hostMapper = hostMapper;
			this.peerMapper = peerMapper;
		}

		protected override bool UseCache {
			get { return false; }
		}

		private readonly HostDatabaseMapper hostMapper;
		private readonly PeerDatabaseMapper peerMapper;

		protected override void Insert (PeerReading item)
		{
			Open ();
			
			Command.CommandText = 
				"INSERT INTO peerReading (time, hostId, peerId, last, reach, delay, offset, jitter) " +
				"VALUES ( ?time, ?hostId, ?peerId, ?last, ?reach, ?delay, ?offset, ?jitter);" +
				"SELECT LAST_INSERT_ID();";
			Command.Prepare ();
			Command.Parameters.AddWithValue ("?time", item.Time);						
			Command.Parameters.AddWithValue ("?hostId", item.Host.Id);
			Command.Parameters.AddWithValue ("?peerId", item.Peer.Id);
			Command.Parameters.AddWithValue ("?last", item.Last);
			Command.Parameters.AddWithValue ("?reach", item.Reach);
			Command.Parameters.AddWithValue ("?delay", item.Delay);
			Command.Parameters.AddWithValue ("?offset", item.Offset);
			Command.Parameters.AddWithValue ("?jitter", item.Jitter);

			object idObject = Command.ExecuteScalar ();
			item.SetId (Convert.ToInt32 (idObject));
		}

		protected override void Update (PeerReading item)
		{
			Open ();
			
			Command.CommandText = 
				"UPDATE peerReading SET " +
				"name = @name, " +
				"hostId = @hostId, " +
				"peerId = @peerId, " +
				"last = @last, " +
				"reach = @reach, " +
				"delay = @delay, " +
				"offset = @offset, " +
				"jitter = @jitter, " +
				"frequency = @frequency, " +
				"stability = @stability " +
				"WHERE id = @id;";
			Command.Prepare ();
           
			Command.Parameters.AddWithValue ("@time", item.Time);
			Command.Parameters.AddWithValue ("@hostId", item.Host.Id);
			Command.Parameters.AddWithValue ("@peerId", item.Peer.Id);
			Command.Parameters.AddWithValue ("@last", item.Last);
			Command.Parameters.AddWithValue ("@reach", item.Reach);
			Command.Parameters.AddWithValue ("@delay", item.Delay);
			Command.Parameters.AddWithValue ("@offset", item.Offset);
			Command.Parameters.AddWithValue ("@jitter", item.Jitter);
			Command.Parameters.AddWithValue ("@id", item.Id);
			Command.ExecuteNonQuery ();
		}

		/// <summary>
		/// Gets or sets the host 
		/// </summary>
		/// <remarks>Always set this property while using other filter properties</remarks>
		/// <value>The host.</value>
		public Host FilterHost { get; set; }

		/// <summary>
		/// Gets or sets the peer to use when extracting data.
		/// </summary>
		/// <value>The filter peer.</value>
		public Peer FilterPeer { get; set; }

		/// <summary>
		/// Gets or sets the time to use when extracting data. Only readings with a timestamp
		/// later than FilterTime gets extracted.
		/// </summary>
		/// <value>The time.</value>
		public DateTime? FilterTime { get; set; }

		/// <summary>
		/// Read all data from table in a sequential manner.
		/// </summary>
		/// <returns>The enumerator.</returns>
		public override System.Collections.Generic.IEnumerator<PeerReading> GetEnumerator ()
		{	
			Open ();
			
			string sql = "SELECT " +
				"id, time, hostId, peerId, last, reach, delay, offset, jitter " +
				"FROM peerReading";
			
			if (FilterHost != null)
				sql += " WHERE hostId = " + FilterHost.Id;
				
			if (FilterPeer != null)
				sql += " AND peerId = " + FilterPeer.Id;

			if (FilterTime != null)
				sql += " AND time > '" + FilterTime.Value.ToString ("yyyy-MM-dd HH:mm") + "'";
								
			Command.CommandText = sql;
			MySqlDataReader reader = Command.ExecuteReader ();
			
			while (reader.Read()) {
				int id = Convert.ToInt32 (reader ["id"]);
				DateTime time = Convert.ToDateTime (reader ["time"]);
				int hostId = Convert.ToInt32 (reader ["hostId"]);
				Host host = hostMapper [hostId];
				int peerId = Convert.ToInt32 (reader ["peerId"]);
				Peer peer = peerMapper [peerId];
				int last = Convert.ToInt32 (reader ["last"]);
				int reach = Convert.ToInt32 (reader ["reach"]);
				double delay = Convert.ToDouble (reader ["delay"]);
				double offset = Convert.ToDouble (reader ["offset"]);
				double jitter = Convert.ToDouble (reader ["jitter"]);

				
				PeerReading reading = new PeerReading (
					id, time, host, peer, null,
					last, reach, delay, offset, jitter);
				
				yield return reading;
			}
			
			Close ();
		}
	}
}
