//
// TimeServerDatabaseMapper.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Net;
using System.Data;
using System.Collections.Generic;
using MySql.Data;
using MySql.Data.MySqlClient;
using Ntp.Analyzer.Objects;
using Ntp.Analyzer.Data.Import;

namespace Ntp.Analyzer.Data.Mapper
{
	/// <summary>
	/// ORM mapper for table timeServer.
	/// </summary>
	public sealed class TimeServerDatabaseMapper : DatabaseMapper<TimeServer>
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="Ntp.Analyzer.Data.Mapper.TimeServerDatabaseMapper"/> class.
		/// </summary>
		internal TimeServerDatabaseMapper ()
			: base()
		{
		}

		protected override bool UseCache {
			get { return true; }
		}

		protected override void Insert (TimeServer item)
		{
			Open ();
			
			Command.CommandText = 
				"INSERT INTO timeServer(id, stratum, countryCode, hostName, ip, ip6, " +
				"useDns, poolMember, location, displayLocation, organization, " +
				"coordinates, synchronization, serviceArea, accessPolicy, " +
				"accessDetails, notification, autoKeyUrl, symmetricKeyType, " +
				"symmetricKeyURL, serverContact, providerPage, providerUrl, " +
				"updated) " +
				"VALUES (?id, ?stratum, ?countryCode, ?hostName, ?ip, ?ip6, " +
				"?useDns, ?poolMember, ?location, ?displayLocation, ?organization, " +
				"?coordinates, ?synchronization, ?serviceArea, ?accessPolicy, " +
				"?accessDetails, ?notification, ?autoKeyUrl, ?symmetricKeyType, " +
				"?symmetricKeyURL, ?serverContact, ?providerPage, ?providerUrl, " +
				"?updated);";
			Command.Prepare ();
           
			Command.Parameters.AddWithValue ("?id", item.Id);
			Command.Parameters.AddWithValue ("?stratum", item.Stratum);
			Command.Parameters.AddWithValue ("?countryCode", item.Country);
			Command.Parameters.AddWithValue ("?hostName", item.Name);
			Command.Parameters.AddWithValue ("?ip", (item.Address != null) ? (object)item.Address.ToString () : DBNull.Value);
			Command.Parameters.AddWithValue ("?ip6", (object)item.V6address ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?useDns", (object)item.ShouldUseDns ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?poolMember", (object)item.IsPoolMember ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?location", (object)item.Location ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?displayLocation", item.DisplayLocation);
			Command.Parameters.AddWithValue ("?organization", (object)item.Organization ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?coordinates", (object)item.Geo ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?synchronization", (object)item.Server ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?serviceArea", (object)item.ServiceArea ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?accessPolicy", (object)item.AccessPolicy ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?accessDetails", (object)item.AccessDetails ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?notification", (object)item.ShouldNotify ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?autoKeyUrl", (object)item.AutoKey ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?symmetricKeyType", (object)item.SymKey ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?symmetricKeyURL", (object)item.SymUrl ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?serverContact", (object)item.Contact ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?providerPage", (object)item.ProviderPage ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?providerUrl", (object)item.ProviderUrl ?? DBNull.Value);
			Command.Parameters.AddWithValue ("?updated", item.Updated);
			
			Command.ExecuteNonQuery ();
			item.SetId (item.Id);
		}

		protected override void Update (TimeServer item)
		{
			throw new NotImplementedException ("TimeServerDatabaseMapper does not implement an update of data");
		}

		/// <summary>
		/// Read all data from table in a sequential manner.
		/// </summary>
		/// <returns>The enumerator.</returns>
		public override IEnumerator<TimeServer> GetEnumerator ()
		{
			if (HasContent) {
				foreach (TimeServer host in Content)
					yield return host;
			
				yield break;
			}
			
			Open ();
			Command.CommandText = "SELECT id, stratum, countryCode, hostName, ip, ip6, " +
				"useDns+0 as dns, poolMember+0 as pool, location, displayLocation, organization, " +
				"coordinates, synchronization, serviceArea, accessPolicy, " +
				"accessDetails, notification+0 as noti, autoKeyUrl, symmetricKeyType, " +
				"symmetricKeyURL, serverContact, providerPage, providerUrl, " +
				"updated " +
				"FROM timeServer;";
			MySqlDataReader reader = Command.ExecuteReader ();

			while (reader.Read()) {
				int id = Convert.ToInt32 (reader ["id"]);
				int stratum = Convert.ToInt32 (reader ["stratum"]);
				string countryCode = reader ["countryCode"].ToString ();
				string hostName = reader ["hostName"].ToString ();
				string ip4 = (reader ["ip"] != DBNull.Value) ? reader ["ip"].ToString () : null;
				string ip6 = (reader ["ip6"] != DBNull.Value) ? reader ["ip6"].ToString () : null;
				bool? useDns = (reader ["dns"] != DBNull.Value) ? Convert.ToBoolean (reader ["dns"]) : (bool?)null;
				bool? poolMember = reader ["pool"] != DBNull.Value ? Convert.ToBoolean (reader ["pool"]) : (bool?)null;
				string location = reader ["location"].ToString ();
				string displayLocation = reader ["displayLocation"].ToString ();
				string organization = (reader ["organization"] != DBNull.Value) ? reader ["organization"].ToString () : null;
				string coordinates = (reader ["coordinates"] != DBNull.Value) ? reader ["organization"].ToString () : null;
				string synchronization = reader ["synchronization"].ToString ();
				string serviceArea = reader ["serviceArea"].ToString ();
				string accessPolicy = reader ["accessPolicy"].ToString ();
				string accessDetails = (reader ["accessDetails"] != DBNull.Value) ? reader ["accessDetails"].ToString () : null;
				bool? notification = (reader ["noti"] != DBNull.Value) ? Convert.ToBoolean (reader ["noti"]) : (bool?)null;
				string autoKeyUrl = (reader ["autoKeyUrl"] != DBNull.Value) ? reader ["autoKeyUrl"].ToString () : null;
				string symmetricKeyType = (reader ["symmetricKeyType"] != DBNull.Value) ? reader ["symmetricKeyType"].ToString () : null;
				string symmetricKeyUrl = (reader ["symmetricKeyURL"] != DBNull.Value) ? reader ["symmetricKeyURL"].ToString () : null;
				string serverContact = (reader ["serverContact"] != DBNull.Value) ? reader ["serverContact"].ToString () : null;
				string providerPage = (reader ["providerPage"] != DBNull.Value) ? reader ["providerPage"].ToString () : null;
				string providerUrl = (reader ["providerUrl"] != DBNull.Value) ? reader ["providerUrl"].ToString () : null;
				DateTime updated = Convert.ToDateTime (reader ["updated"]);
				
				IPAddress address = null;
				if (ip4 != null)
					IPAddress.TryParse (ip4, out address);
				
				TimeServer server = new TimeServer (
					id, stratum, countryCode, hostName, address, ip6, useDns, poolMember, location,
					displayLocation, organization, coordinates, synchronization, serviceArea, accessDetails,
					accessPolicy, notification, autoKeyUrl, symmetricKeyType, symmetricKeyUrl, serverContact,
					providerPage, providerUrl, updated);
				
				AddItem (server);
				
				yield return server;
			}
			
			Close ();
		}

		/// <summary>
		/// Fetch time server from external source.
		/// </summary>
		/// <returns>The external.</returns>
		/// <param name="id">The time server identifier used on support.ntp.org.</param>
		protected override TimeServer FetchExternal (int id)
		{
			TimeServerLoader loader = new TimeServerLoader ();
			TimeServerImporter importer = new TimeServerImporter ();
			
			string table = loader.ImportServer (id);
			TimeServer server = importer.ParseTable (table, id);
			
			Insert (server);
			AddItem (server);
			
			return server;
		}
	}
}
