//
// PeerReadingDatabaseMapper.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.net>
//
// Copyright (c) 2014 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using Ntp.Analyzer.Objects;

namespace Ntp.Analyzer.Data.Static
{
    /// <summary>
    /// ORM mapper for table peerReading.
    /// </summary>
    public sealed class PeerReadingDatabaseMapper : FilteredMapper<PeerReading>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PeerReadingDatabaseMapper"/> class.
        /// </summary>
        /// <param name="hostMapper">Host mapper.</param>
        /// <param name="peerMapper">Peer mapper.</param>
        internal PeerReadingDatabaseMapper(
            HostDatabaseMapper hostMapper,
            PeerDatabaseMapper peerMapper)
        {
            this.hostMapper = hostMapper;
            this.peerMapper = peerMapper;
        }

        private readonly HostDatabaseMapper hostMapper;
        private readonly PeerDatabaseMapper peerMapper;

        protected override bool UseCache
        {
            get { return false; }
        }

        private const string SelectSql =
            "SELECT " +
            "id, time, hostId, peerId, last, reach, delay, offset, jitter " +
            "FROM peerReading";

        private const string InsertSql =
            "INSERT INTO peerReading (time, hostId, peerId, last, reach, delay, offset, jitter) " +
            "VALUES ( ?time, ?hostId, ?peerId, ?last, ?reach, ?delay, ?offset, ?jitter);" +
            "SELECT LAST_INSERT_ID();";

        private const string UpdateSql =
            "UPDATE peerReading SET " +
            "name = @name, " +
            "hostId = @hostId, " +
            "peerId = @peerId, " +
            "last = @last, " +
            "reach = @reach, " +
            "delay = @delay, " +
            "offset = @offset, " +
            "jitter = @jitter, " +
            "frequency = @frequency, " +
            "stability = @stability " +
            "WHERE id = @id;";

        protected override void Insert(PeerReading item)
        {
            lock (Locker)
            {
                try
                {
                    Open();

                    Command.CommandText = InsertSql;
                    Command.Prepare();
                    Command.Parameters.AddWithValue("?time", item.Time);
                    Command.Parameters.AddWithValue("?hostId", item.Host.Id);
                    Command.Parameters.AddWithValue("?peerId", item.Peer.Id);
                    Command.Parameters.AddWithValue("?last", item.Last);
                    Command.Parameters.AddWithValue("?reach", item.Reach);
                    Command.Parameters.AddWithValue("?delay", item.Delay);
                    Command.Parameters.AddWithValue("?offset", item.Offset);
                    Command.Parameters.AddWithValue("?jitter", item.Jitter);

                    object idObject = Command.ExecuteScalar();
                    item.SetId(Convert.ToInt32(idObject));
                }
                finally
                {
                    Close();
                }
            }
        }

        protected override void Update(PeerReading item)
        {
            lock (Locker)
            {
                try
                {
                    Open();

                    Command.CommandText = UpdateSql;
                    Command.Prepare();

                    Command.Parameters.AddWithValue("@time", item.Time);
                    Command.Parameters.AddWithValue("@hostId", item.Host.Id);
                    Command.Parameters.AddWithValue("@peerId", item.Peer.Id);
                    Command.Parameters.AddWithValue("@last", item.Last);
                    Command.Parameters.AddWithValue("@reach", item.Reach);
                    Command.Parameters.AddWithValue("@delay", item.Delay);
                    Command.Parameters.AddWithValue("@offset", item.Offset);
                    Command.Parameters.AddWithValue("@jitter", item.Jitter);
                    Command.Parameters.AddWithValue("@id", item.Id);
                    Command.ExecuteNonQuery();
                }
                finally
                {
                    Close();
                }
            }
        }

        /// <summary>
        /// Read all data from table in a sequential manner.
        /// </summary>
        /// <returns>The enumerator.</returns>
        public override IEnumerator<PeerReading> GetEnumerator()
        {
            lock (Locker)
            {
                string sql = SelectSql;

                if (FilterHost != null)
                    sql += " WHERE hostId = " + FilterHost.Id;

                if (FilterPeer != null)
                    sql += " AND peerId = " + FilterPeer.Id;

                if (FilterTime != null)
                    sql += " AND time > '" + FilterTime.Value.ToString("yyyy-MM-dd HH:mm") + "'";

                try
                {
                    Open();
                    Command.CommandText = sql;
                    Reader = Command.ExecuteReader();

                    while (Reader.Read())
                    {
                        int id = Convert.ToInt32(Reader["id"]);
                        DateTime time = Convert.ToDateTime(Reader["time"]);
                        int hostId = Convert.ToInt32(Reader["hostId"]);
                        Host host = hostMapper[hostId];
                        int peerId = Convert.ToInt32(Reader["peerId"]);
                        Peer peer = peerMapper[peerId];
                        int last = Convert.ToInt32(Reader["last"]);
                        int reach = Convert.ToInt32(Reader["reach"]);
                        double delay = Convert.ToDouble(Reader["delay"]);
                        double offset = Convert.ToDouble(Reader["offset"]);
                        double jitter = Convert.ToDouble(Reader["jitter"]);

                        PeerReading reading = new PeerReading(
                            id, time, host, peer, null,
                            last, reach, delay, offset, jitter);

                        yield return reading;
                    }
                }
                finally
                {
                    Close();
                }
            }
        }
    }
}
