//
// LogConfiguration.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.net>
//
// Copyright (c) 2014 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using Ntp.Config;

namespace Ntp.Analyzer.Log
{
    public sealed class LogConfiguration : ConfigurationBase
    {
        public const string Identifier = "Log";

        private static readonly String[] Params = new[]
            {
                Identifier,
                "Severity",
                "File"
            };

        private readonly string file;
        private readonly Severity treshold;

        private LogConfiguration(string configName, int indentLevel, string file, Severity treshold)
            : base(configName, indentLevel)
        {
            this.file = file;
            this.treshold = treshold;
        }

        public string File
        {
            get { return file; }
        }

        public Severity Treshold
        {
            get { return treshold; }
        }

        protected override IEnumerable<Object> Items
        {
            get
            {
                return new object[]
                    {
                        ConfigName,
                        ConvertFromSeverity(Treshold),
                        File
                    };
            }
        }

        public override string ToString()
        {
            return ConfigText(Params, Items);
        }

        public static LogConfiguration Create(ConfigurationBlock config)
        {
            Dictionary<String, String> options = GetOptions(Params, config);

            return new LogConfiguration(
                GetOptionConfigName(options, Params[0], String.Empty, false),
                config.IndentLevel,
                GetOptionString(options, Params[2], "Log", true),
                ConvertToSeverity(options[Params[1]])
                );
        }

        private static Severity ConvertToSeverity(string text)
        {
            switch (text.ToLower())
            {
                case "error":
                    return Severity.Error;
                case "warn":
                    return Severity.Warn;
                case "notice":
                    return Severity.Notice;
                case "info":
                    return Severity.Info;
                case "debug":
                    return Severity.Debug;
                default:
                    throw new ArgumentException("Unknown severity type: " + text);
            }
        }

        private static string ConvertFromSeverity(Severity severity)
        {
            switch (severity)
            {
                case Severity.Error:
                    return "error";
                case Severity.Warn:
                    return "warn";
                case Severity.Notice:
                    return "notice";
                case Severity.Info:
                    return "info";
                case Severity.Debug:
                    return "debug";
                default:
                    throw new ArgumentException("Unknown severity type: " + severity);
            }
        }
    }
}
