//
// TextLog.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.net>
//
// Copyright (c) 2014 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.IO;
using System.Text;

namespace Ntp.Analyzer.Log
{
    public sealed class TextLog : LogBase
    {
        private readonly string file;
        private readonly Object locker = new Object();
        private readonly string timeFormat;
        private readonly Severity treshold;
        private bool initialized;
        private TextWriter writer;

        public TextLog(string file, Severity treshold, string timeFormat)
        {
            this.file = file;
            this.treshold = treshold;
            this.timeFormat = timeFormat;
            initialized = false;
        }

        public TextLog(string file, Severity treshold)
            : this(file, treshold, "yyyy-MM-dd HH:mm:ss")
        {
        }

        private void Initialize()
        {
            writer = new StreamWriter(file, true, Encoding.ASCII);
            initialized = true;
        }

        public override void WriteLine(string text, Severity severity)
        {
            if (!initialized)
                Initialize();

            if (severity < treshold)
                return;

            string severityText;

            switch (severity)
            {
                case Severity.Error:
                    severityText = "ERROR";
                    break;
                case Severity.Warn:
                    severityText = "WARN";
                    break;
                case Severity.Notice:
                    severityText = "NOTICE";
                    break;
                case Severity.Info:
                    severityText = "INFO";
                    break;
                case Severity.Debug:
                    severityText = "DEBUG";
                    break;
                default:
                    throw new ApplicationException("Unknown severity level.");
            }

            DateTime now = DateTime.Now;

            string entry = String.Concat(
                now.ToString(timeFormat),
                " [", severityText, "] ",
                text);

            lock (locker)
            {
                writer.WriteLine(entry);
                writer.Flush();
            }
        }

        public override void WriteLine(Exception exception)
        {
            WriteLine(exception.Message, Severity.Error);
            WriteLine(exception.StackTrace, Severity.Debug);
        }

        public void Close()
        {
            writer.Close();
            initialized = false;
        }
    }
}
