//
// JobBase.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using Ntp.Analyzer.Log;
using Ntp.Analyzer.Config;

namespace Ntp.Analyzer.Process.Job
{
    /// <summary>
    /// Base class for jobs following the GoF Command Pattern.
    /// </summary>
    public abstract class JobBase
    {
        private static Object nextJobLocker = new object ();
        private static int nextJobId = 1;

        protected JobBase (ScheduledJobConfiguration config, LogBase log)
        {
            this.config = config;
            this.log = log;
            this.runCount = 0;
			
            lock (nextJobLocker) {
                this.jobId = nextJobId++;
            }
        }

        private readonly ScheduledJobConfiguration config;
        private readonly LogBase log;
        private readonly int jobId;
        private int runCount;

        protected LogBase Log {
            get { return log; }
        }

        public abstract string Name { get; }

        public abstract int Priority { get; }

        /// <summary>
        /// Gets the number of times this job have been executed.
        /// </summary>
        /// <value>The run count.</value>
        public int RunCount {
            get { return runCount; }
        }

        public string Description {
            get {
                return String.Format (
                    "Job {0}, {1}, config {2}, initial run {3}, frequency {4}", 
                    jobId.ToString ("00"), Name, config.ConfigName, InitialRun, Frequency);
            }
        }

        public ScheduledJobConfiguration ScheduledConfig {
            get { return config; }
        }

        public bool InitialRun {
            get { return config.InitialRun; }
        }

        public int Frequency { 
            get { return config.Frequency; }
        }

        /// <summary>
        /// Execute this instance.
        /// </summary>
        public void Execute ()
        {
            runCount++;
			
            InternalExecute ();
        }

        protected abstract void InternalExecute ();

        public override string ToString ()
        {
            return String.Format ("Job {0}, {1}, config {2}", jobId.ToString ("00"), Name, config.ConfigName);
        }
    }
}
