//
// NotifyJob.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using System.Linq;
using System.Text;
using System.Net.Mail;
using System.Collections.Generic;
using Ntp.Analyzer.Log;
using Ntp.Analyzer.Config;

namespace Ntp.Analyzer.Process.Job
{
    public sealed class NotifyJob : JobBase
    {
        public NotifyJob (
            NotifyConfiguration config,
            LogBase log,
            int pid,
            string configFile
        )
			: base(config, log)
        {
            this.config = config;
            this.pid = pid;
            this.configFile = configFile;
            jobs = new List<JobBase> ();
			
            client = new SmtpClient (config.SmptServer, config.SmtpPort);
        }

        private readonly NotifyConfiguration config;
        private readonly string configFile;
        private readonly int pid;
        private readonly List<JobBase> jobs;
        private readonly SmtpClient client;

        public override string Name {
            get { return "Notify mail"; }
        }

        public override int Priority {
            get { return 99;	}
        }

        /// <summary>
        /// Sets the job list to be monitored.
        /// </summary>
        /// <param name="scheduledJobs">Scheduled jobs.</param>
        internal void SetJobList (ScheduledJob[] scheduledJobs)
        {
            jobs.AddRange (scheduledJobs.Select (j => j.Job));
        }

        protected override void InternalExecute ()
        {
            MailMessage message = new MailMessage (
                config.Mail,
                config.Sender,
                config.Subject,
                BuildContent ()
            );
			
            client.Send (message);
        }

        /// <summary>
        /// Builds the message content.
        /// </summary>
        /// <returns>The content.</returns>
        private string BuildContent ()
        {
            StringBuilder builder = new StringBuilder ();
			
            builder.AppendLine ("List of running jobs in NTP Analyzer");
			
            foreach (JobBase job in jobs) {
                builder.Append (job.Description);
                builder.Append (": ");
                builder.AppendLine (job.RunCount.ToString ());
            }

            builder.Append ("Using configuration: ");
            builder.AppendLine (configFile);
			
            builder.Append ("Running with PID: ");
            builder.AppendLine (pid.ToString ());
			
            return builder.ToString ();
        }
    }
}
