//
// PeerPageJob.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using System.IO;
using System.Linq;
using System.Collections.Generic;
using Ntp.Analyzer.Log;
using Ntp.Analyzer.Data;
using Ntp.Analyzer.Page;
using Ntp.Analyzer.Objects;
using Ntp.Analyzer.Config.Page;
using Ntp.Analyzer.Config.Navigation;

namespace Ntp.Analyzer.Process.Job
{
    public sealed class PeerPageJob : StreamJob
    {
        public PeerPageJob (
            MenuConfiguration menu,
            PeerPageConfiguration page,
            LogBase log)
            : base(page, log)
        {
            this.menu = menu;
            this.page = page;
        }

        private readonly MenuConfiguration menu;
        private readonly PeerPageConfiguration page;

        public override string Name {
            get { return "Peer pages build"; }
        }

        public override int Priority {
            get { return 11; }
        }

        protected override void InternalExecute ()
        {
            Host host = DataFace.Instance.Hosts.Single (h => h.Id == page.HostId);

            foreach (NtpConfEntry entry in DataFace.Instance.NtpConfigCache[page.ConfigFile]) {

                IEnumerable<Peer> peers = DataFace.Instance.Peers.Where (p => p.OrgId == entry.OrgId.Value);
				
                if (peers.Count () == 0) {
                    Log.WriteLine ("Peer not found in database. Creating peer: " + entry.Name, Severity.Warn);

                    TimeServer server = DataFace.Instance.Servers [entry.OrgId.Value];
                    Peer newPeer = new Peer (entry.Name, entry.Address.ToString (), server);
                    DataFace.Instance.Peers.Save (newPeer);
					
                    peers = (new List<Peer> () { newPeer }).ToList ();
					
                } else if (peers.Count () > 1) {
                    Log.WriteLine ("Ambiguous peer name. Multiple occurrence found in database: " + entry.Name, Severity.Warn);
                    continue;
                }

                Peer peer = peers.Single ();
				
                PageBuilderBase peerPageBuilder;

                if (page.Theme == "bootstrap")
                    peerPageBuilder = new BootstrapPeerPageBuilder (host, peer, menu, page);
                else
                    peerPageBuilder = new DefaultPeerPageBuilder (peer, page);

                SaveStream (peerPageBuilder, peer.OrgId.ToString () + ".html");
            }
        }
    }
}
