//
// Runner.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.IO;
using System.Threading;
using System.Globalization;
using Ntp.Analyzer.Log;
using Ntp.Analyzer.Data;
using Ntp.Analyzer.Config;

namespace Ntp.Analyzer.Process
{
    public sealed class Runner
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Ntp.Analyzer.Process.Runner"/> class.
        /// </summary>
        /// <param name="configFile">Config file to use.</param>
        /// <param name="pid">Process ID. Only used for logging.</param>
        public Runner (string configFile, int pid)
        {
            // Also update solution version.
            version = "8.4.2";
		
            this.configFile = configFile;
            this.pid = pid;
            this.scheduler = new Scheduler ();
        }

        private readonly string version;
        private readonly string configFile;
        private readonly int pid;
        private readonly Scheduler scheduler;

        /// <summary>
        /// Run the NTP Analyzer.
        /// </summary>
        public void Run ()
        {
            // Neutralize.
            Thread.CurrentThread.CurrentCulture = CultureInfo.InvariantCulture;

            // Read config.
            string[] configLines = File.ReadAllLines (configFile);
            Configuration config = Configuration.Create (configLines);

            LogBase log = LogFactory.CreateLog(config.Log); 
            DataConnection.String = config.Database.ConnectionString;
			
            // Initialize log.
            log.WriteLine ("NTP Analyzer " + version + " started.", Severity.Notice);
            log.WriteLine ("Using configuration: " + configFile, Severity.Notice);
            log.WriteLine ("Running with pid: " + pid, Severity.Notice);
			
            // Initialize scheduler.
            try {
                scheduler.Initialize (log, pid, configFile, config);
            } catch (Exception e) {
                log.WriteLine ("Unexpected error during initialization.", Severity.Error);
                log.WriteLine (e.ToString (), Severity.Error);
            }
			
            // Go !
            try {
                scheduler.Run ();
            } catch (Exception e) {
                log.WriteLine ("Unexpected error during run.", Severity.Error);
                log.WriteLine (e.ToString (), Severity.Error);
            }
        }
    }
}
