//
// Scheduler.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
using System;
using System.IO;
using System.Linq;
using System.Threading;
using System.Globalization;
using System.Collections.Generic;
using Ntp.Analyzer.Log;
using Ntp.Analyzer.Data;
using Ntp.Analyzer.Process.Job;
using Ntp.Analyzer.Config;
using Ntp.Analyzer.Config.Page;
using Ntp.Analyzer.Config.Graph;

namespace Ntp.Analyzer.Process
{
    internal sealed class Scheduler
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="Ntp.Analyzer.Process.Scheduler"/> class.
        /// </summary>
        internal Scheduler ()
        {
            this.startupTime = DateTime.Now;	
            schedule = new List<ScheduledJob> ();
        }

        private readonly DateTime startupTime;
        private readonly List<ScheduledJob> schedule;
        private LogBase log;

        /// <summary>
        /// Initialize the scheduler including all its containing jobs.
        /// </summary>
        /// <param name="log">Log.</param>
        /// <param name="pid">Pid.</param>
        /// <param name="configFile">Config file.</param>
        /// <param name="config">Config.</param>
        internal void Initialize (LogBase log, int pid, string configFile, Configuration config)
        {
            this.log = log;

            log.WriteLine ("Initializing jobs.", Severity.Debug);

            InitializeJob (new BulkStatJob (config.Bulk, log));

            // Add jobs to schedule.
            foreach (ServerConfiguration server in config.Servers) {
                InitializeJob (new HostStatJob (server.HostStats, log));
                InitializeJob (new PeerStatJob (server.PeerStats, log));
                InitializeJob (new TcpStatJob (server.TcpdumpStats, log));

                InitializeJob (new AboutPageJob (server.Menu, server.AboutPage, log));

                foreach (PeerPageConfiguration peerPage in server.PeerPages)
                    InitializeJob (new PeerPageJob (server.Menu, peerPage, log));
				
                foreach (HostPageConfiguration hostPage in server.HostPages)
                    InitializeJob (new HostPageJob (server.Menu, hostPage, log));

                foreach (HostGraphConfiguration hostGraph in server.HostGraphs)
                    InitializeJob (new HostGraphJob (hostGraph, log));
				
                foreach (PeerGraphConfiguration peerGraph in server.PeerGraphs)
                    InitializeJob (new PeerGraphJob (peerGraph, log));

                foreach (PeerSummeryPageConfiguration peerSummeryPage in server.PeerSummeryPages)
                    InitializeJob (new PeerSummeryJob (server.Menu, peerSummeryPage, log));
            
                 InitializeJob (new HostGraphPageJob (server.HostGraphPage, log));
                 InitializeJob (new PeerGraphPageJob (server.PeerGraphPage, log));
            }
			
            // Add the survaliance/notify job.
            NotifyJob adminJob = new NotifyJob (config.Notify, log, pid, configFile);
            InitializeJob (adminJob);
            adminJob.SetJobList (schedule.ToArray ());
			
            // Done
            log.WriteLine ("All jobs initialized.", Severity.Debug);
        }

        /// <summary>
        /// Run the scheduler queue pump method.
        /// </summary>
        internal void Run ()
        {
            Thread.CurrentThread.CurrentCulture = CultureInfo.InvariantCulture;
					
            log.WriteLine (String.Format ("Starting scheduler with {0} jobs.", schedule.Count ()), Severity.Notice);
			
            while (true) {
                ScheduledJob next = schedule.OrderBy (j => j.Run).ThenBy (j => j.Job.Priority).First ();
                schedule.Remove (next);
				
                int wait = Convert.ToInt32 (next.Run.Subtract (DateTime.Now).TotalMilliseconds);
				
                if (wait < 0) {
                    if (wait < -5000)
                        log.WriteLine ("Behind schedule. Trying to catch up.", Severity.Info);
					
                    wait = 0;
                }
				
                log.WriteLine (next.ToString (), Severity.Info); 

                Thread.Sleep (wait);
			
                log.WriteLine ("Executing " + next.Job, Severity.Debug);
				
                try {
                    next.Job.Execute ();
                } catch (Exception e) {
                   log.WriteLine(e);
                }
				
                // Dont re-schedule "run-only-once" jobs.
                if (next.Job.Frequency == 0)
                    continue;

                DateTime run = Math.Schedule.CalculateNextRun (DateTime.Now, next.Job.Frequency);
                QueueJob (next.Job, run);
            }
        }

        /// <summary>
        /// Initializes the job with its default parameters.
        /// </summary>
        /// <param name="job">Job.</param>
        private void InitializeJob (JobBase job)
        {
            if (job.ScheduledConfig == null || job.ScheduledConfig.Frequency == -1)
                return;
		
            DateTime run = job.InitialRun ? startupTime : Math.Schedule.CalculateNextRun (DateTime.Now, job.Frequency);
						
            QueueJob (job, run);
			
            log.WriteLine ("Job Initialized: " + job.Description, Severity.Debug);
        }

        private readonly Object locker = new Object ();

        /// <summary>
        /// Queue job for scheduled run.
        /// </summary>
        /// <param name="job">Job.</param>
        /// <param name="run">Run.</param>
        private void QueueJob (JobBase job, DateTime run)
        {
            lock (locker) {
                ScheduledJob scheduledJob = new ScheduledJob (job, run);
                schedule.Add (scheduledJob);
				
                log.WriteLine ("Scheduled " + scheduledJob, Severity.Debug);
            }
        }
    }
}
