//
// GraphSet.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.net>
//
// Copyright (c) 2014 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using Ntp.Analyzer.Config.Graph;
using Ntp.Config;

namespace Ntp.Analyzer.Config.Destination
{
    public sealed class GraphSetConfiguration : ConfigurationBase
    {
        public const string Identifier = "Images";

        private static readonly String[] Params = new[]
            {
                Identifier,
                "Title",
                "LinkIndex"
            };

        private readonly List<GraphBaseConfiguration> graphs;
        private readonly int linkIndex;
        private readonly string title;

        public GraphSetConfiguration(
            string configName,
            int indentLevel,
            string title,
            int linkIndex,
            IEnumerable<GraphBaseConfiguration> graphs)
            : base(configName, indentLevel)
        {
            this.title = title;
            this.linkIndex = linkIndex;
            this.graphs = new List<GraphBaseConfiguration>(graphs);
        }

        public string Title
        {
            get { return title; }
        }

        public int LinkIndex
        {
            get { return linkIndex; }
        }

        public IEnumerable<GraphBaseConfiguration> Graphs
        {
            get { return graphs; }
        }

        protected override IEnumerable<Object> Items
        {
            get
            {
                return new object[]
                    {
                        ConfigName,
                        Title,
                        LinkIndex
                    };
            }
        }

        public override string ToString()
        {
            List<Object> values = new List<Object>(Items);

            IEnumerable<String> names = Graphs.Select(g => g.ConfigName);
            values.AddRange(names);

            List<String> settings = new List<string>(Params);

            int num = names.Count();

            for (int i = 0; i < num; i++)
                settings.Add("Graph");

            return ConfigText(settings.ToArray(), values.ToArray());
        }

        public static IEnumerable<GraphSetConfiguration> Create(
            ServerConfiguration server,
            IEnumerable<GraphBaseConfiguration> graphs,
            ConfigurationBlock config,
            string configPath)
        {
            List<GraphSetConfiguration> sets = new List<GraphSetConfiguration>();

            foreach (ConfigurationBlock imageBlock in FindBlocks("Images", config))
            {
                Dictionary<String, String> options = GetOptions(Params, imageBlock);
                string title = GetOptionString(options, "Title", configPath, false);
                int linkIndex = GetOptionInteger(options, "LinkIndex", configPath, true);

                List<GraphBaseConfiguration> selectedGraphs = new List<GraphBaseConfiguration>();
                foreach (string name in GetMultiOption("Graph", imageBlock))
                {
                    selectedGraphs.Add(graphs.Single(p => p.GraphName == name));
                }

                sets.Add(new GraphSetConfiguration(String.Empty, config.IndentLevel + 1, title, linkIndex, selectedGraphs));
            }

            return sets;
        }
    }
}