//
// MenuItemConfiguration.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.net>
//
// Copyright (c) 2014 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using Ntp.Config;

namespace Ntp.Analyzer.Config.Navigation
{
    public abstract class MenuItemConfiguration : ConfigurationBase
    {
        private static readonly String[] Params = new[]
            {
                "Item",
                "Type",
                "Caption",
                "Link",
                "Page",
                "Dropdown"
            };

        private readonly string caption;
        private readonly string link;

        protected MenuItemConfiguration(
            string configName,
            int indentLevel,
            string caption,
            string link)
            : base(configName, indentLevel)
        {
            this.caption = caption;
            this.link = link;
        }

        public string Caption
        {
            get { return caption; }
        }

        public abstract ILinkable Page { get; }

        public virtual string Link
        {
            get { return link; }
        }

        public abstract string TypeText { get; }

        protected override IEnumerable<Object> Items
        {
            get
            {
                return new object[]
                    {
                        ConfigName,
                        TypeText,
                        Caption,
                        Link,
                        Page != null ? Page.Name : null,
                        null
                    };
            }
        }

        public override string ToString()
        {
            return ConfigText(Params, Items);
        }

        public static MenuItemConfiguration Create(ConfigurationBlock config, IEnumerable<ILinkable> linkables)
        {
            Dictionary<String, String> options = GetOptions(Params, config);

            string itemName = GetOptionConfigName(options, "Item", "Item", true);
            string type = GetOptionString(options, "Type", "Item", false, "Link").ToLower();
            string caption = GetOptionString(options, "Caption", "Item", true);
            string link = GetOptionString(options, "Link", "Item", true);
            string page = GetOptionString(options, "Page", "Item", false);
            string dropdown = GetOptionString(options, "Dropdown", "Item", false);
            ILinkable linkable = null;

            if (dropdown != String.Empty)
                type = "dropdown";
            else if (page != String.Empty)
                type = "page";
            else if (type == String.Empty && link != String.Empty)
                type = "link";

            if (type == "page")
            {
                linkable = linkables.Single(l => l.Name == page);
            }

            switch (type)
            {
                case "spacer":
                    return new SpacerMenuItemConfiguration(String.Empty, config.IndentLevel);
                case "header":
                    return new HeaderMenuItemConfiguration(String.Empty, config.IndentLevel, itemName, caption);
                case "name":
                    return new HeadMenuItemConfiguration(String.Empty, config.IndentLevel, itemName, caption, link);
                case "link":
                    return new LinkMenuItemConfiguration(String.Empty, config.IndentLevel, itemName, caption, link);
                case "page":
                    return new PageMenuItemConfiguration(String.Empty, config.IndentLevel, caption, linkable);
                case "dropdown":
                    return DropdownItemConfiguration.Create(
                        itemName,
                        dropdown,
                        FindSingleBlock("Content", config),
                        linkables);
                default:
                    return null;
            }
        }
    }
}