//
// TcpdumpImporter.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.dk>
//
// Copyright (c) 2013 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.IO;
using System.Net;
using System.Linq;
using System.Collections.Generic;
using Ntp.Analyzer.Objects;
using Ntp.Analyzer.Objects.Packet;
using Ntp.Analyzer.Data;

namespace Ntp.Analyzer.Import
{
	public sealed class TcpdumpImporter
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="Ntp.Analyzer.Import.TcpdumpImporter"/> class.
		/// </summary>
		/// <param name="lastDump">Last dump.</param>
		public TcpdumpImporter (DateTime lastDump, string file)
		{
			this.lastDump = lastDump;
			this.file = file;
		}

		private DateTime lastDump;
		private readonly string file;
		private readonly List<NtpPacket> lines = new List<NtpPacket> ();

		/// <summary>
		/// Gets the time of last imported dump.
		/// </summary>
		/// <value>The last dump.</value>
		public DateTime LastDump {
			get { return lastDump; }
		}

		/// <summary>
		/// Gets the imported tcpdump lines.
		/// </summary>
		/// <value>The lines.</value>
		public NtpPacket[] Lines {
			get { return lines.ToArray (); }
		}

		/// <summary>
		/// Import data from the specified server.
		/// </summary>
		/// <param name='server'>
		/// Server to import from.
		/// </param>
		public void Import ()
		{
			System.Diagnostics.Process proc = new System.Diagnostics.Process ();
			proc.EnableRaisingEvents = false; 
			proc.StartInfo.UseShellExecute = false;
			proc.StartInfo.RedirectStandardOutput = true;
			proc.StartInfo.FileName = "tcpdump";
			proc.StartInfo.WorkingDirectory = "/tmp/";
			proc.StartInfo.Arguments = String.Format (" -nr {0} port 123", file);
			proc.Start ();
			ReadFromStream (proc.StandardOutput);
			
			DataFace.Instance.SourcePeers.CleanUp ();
			DataFace.Instance.NtpPackets.CleanUp ();
		}

		/// <summary>
		/// Reads tcpdump lines in text format from a stream.
		/// </summary>
		/// <param name="reader">Reader.</param>
		private void ReadFromStream (StreamReader reader)
		{
			NtpPacket packet;
									
			while (reader.Peek() != -1) {
				string line = reader.ReadLine ();
			
				if (line.Trim () == String.Empty)
					continue;

				if (line.Substring (16, 2) == "IP")
					packet = ParseSimpleLine (line);
				else
					packet = ParseComplexLine (line);
				
				if (packet != null) {
					lines.Add (packet);
					DataFace.Instance.NtpPackets.Save (packet);
				} else {
					Console.WriteLine ("Cannot create NtpPacket for tcpdump line:");
					Console.WriteLine (line);
				}
			}
		}

		/// <summary>
		/// Parses the simple version of a tcpdump line.
		/// </summary>
		/// <example>
		/// ParseSimpleLine("22:56:01.491229 IP 188.180.108.245.22648 > 95.166.204.165.123: NTPv4, Client, length 48");
		/// </example>
		/// <remarks>
		/// Sample lines:
		/// 22:55:58.253183 IP 83.92.69.104.123 > 95.166.204.165.123: NTPv3, Client, length 48
		/// 22:55:59.040971 IP 2.111.98.138.1024 > 95.166.204.165.123: NTPv3, Client, length 48
		/// 22:55:59.637210 IP 92.72.2.243.123 > 95.166.204.165.123: NTPv3, Client, length 48
		/// 22:55:59.683454 IP 90.185.75.160.123 > 95.166.204.165.123: NTPv4, Client, length 48
		/// 22:55:59.691949 IP 80.71.129.48.55507 > 95.166.204.165.123: NTPv3, Client, length 48
		/// 22:55:59.836963 IP 86.52.150.33.61396 > 95.166.204.165.123: NTPv4, Client, length 48
		/// 22:56:00.811737 IP 89.239.233.185.2532 > 95.166.204.165.123: NTPv3, Client, length 48
		/// 22:56:00.887483 IP 83.161.215.242.61402 > 95.166.204.165.123: NTPv3, Client, length 48
		/// 22:56:01.089959 IP 80.63.2.226.60926 > 95.166.204.165.123: NTPv4, Client, length 48
		/// 22:56:01.124963 IP 93.161.164.47.55128 > 95.166.204.165.123: NTPv4, Client, length 48
		/// 22:56:01.491229 IP 188.180.108.245.22648 > 95.166.204.165.123: NTPv4, Client, length 48
		/// </remarks>
		/// <param name="line">Line.</param>
		/// <returns>The tcpdump line as an object.</returns>
		private NtpPacket ParseSimpleLine (string line)
		{
			string time = line.Substring (0, 15);
			
			DateTime lineTime = ParseTime (time);
			
			// Only import newer dumps
			if (lineTime < lastDump)
				return null;
			
			string source = line.Substring (19, line.IndexOf (' ', 19) - 19);
			
			int destPos = 19 + source.Length + 3;
			string destination = line.Substring (destPos, line.IndexOf (' ', destPos) - destPos - 1);
			
			int versionPos = destPos + destination.Length + 2;
			int versionLength = line.IndexOf (' ', versionPos + 1) - versionPos - 1;
			string version = versionLength > 0 ? line.Substring (versionPos, versionLength) : String.Empty;

			int typePos = versionPos + version.Length + 2;
			int typeLength = line.IndexOf (',', typePos) - typePos;
			
			string type = typeLength > 0 ? line.Substring (typePos, typeLength) : String.Empty;
			
			NtpMajorVersion ntpVersion = ParseVersion (version);
			NtpPacketType packetType = ParsePacketType (type);
			
			if (ntpVersion == NtpMajorVersion.UnknownVersion || packetType == NtpPacketType.UnknownPacket)
				Console.WriteLine (line);
			
			lastDump = lineTime;
			
			return CreatePacket (
				lineTime,
				ParseEndPoint (source),
				ParseEndPoint (destination),
				ntpVersion,
				packetType
			);
		}

		/// <summary>
		/// Parses the complex version of a tcpdump line.
		/// </summary>
		/// <returns>The tcpdump line.</returns>
		/// <example>
		/// ParseComplexLine("20:01:34.629298 93.161.58.210.1024 > 95.166.204.208.123: v3 sym_act strat 2 poll 15 prec -6");
		/// </example>
		/// <remarks>
		/// Sample lines:
		/// 20:01:34.629563 95.166.204.208.123 > 93.161.58.210.1024: v3 sym_pas strat 2 poll 15 prec -18 [tos 0xc0]
		/// 20:01:36.570589 80.167.203.20.62020 > 95.166.204.208.123: v4 client strat 3 poll 4 prec -20
		/// 20:01:36.570856 95.166.204.208.123 > 80.167.203.20.62020: v4 server strat 2 poll 4 prec -18 [tos 0xc0]
		/// 20:01:36.582813 83.88.100.34.123 > 95.166.204.208.123: v4 client strat 3 poll 4 prec -19 [tos 0xc0]
		/// 20:01:36.583046 95.166.204.208.123 > 83.88.100.34.123: v4 server strat 2 poll 4 prec -18 [tos 0xc0]
		/// 20:01:37.298103 87.145.122.42.123 > 95.166.204.208.123: v3 client strat 3 poll 10 prec -19 (DF)
		/// 20:01:37.298897 95.166.204.208.123 > 87.145.122.42.123: v3 server strat 2 poll 10 prec -18 [tos 0xc0]
		/// 20:01:38.341588 80.70.1.244.123 > 95.166.204.208.123: v4 alarm client strat 2 poll 6 prec -21 (DF) [tos 0xc0]
		/// 20:01:38.341894 95.166.204.208.123 > 80.70.1.244.123: v4 server strat 2 poll 6 prec -18 [tos 0xc0]
		/// 20:01:38.531131 87.55.111.34.39302 > 95.166.204.208.123: v4 client strat 3 poll 4 prec -20 [tos 0xc0]
		/// 20:01:38.531442 95.166.204.208.123 > 87.55.111.34.39302: v4 server strat 2 poll 4 prec -18 [tos 0xc0]
		/// 20:01:38.569837 80.167.203.20.62020 > 95.166.204.208.123: v4 client strat 3 poll 4 prec -20
		/// 20:01:38.570075 95.166.204.208.123 > 80.167.203.20.62020: v4 server strat 2 poll 4 prec -18 [tos 0xc0]
		/// 20:01:38.572610 93.164.227.98.123 > 95.166.204.208.123: v4 client strat 3 poll 4 prec -19
		/// 20:01:38.572862 95.166.204.208.123 > 93.164.227.98.123: v4 server strat 2 poll 4 prec -18 [tos 0xc0]
		/// 20:01:38.575821 87.55.101.120.123 > 95.166.204.208.123: v4 client strat 3 poll 4 prec -20
		/// </remarks>
		/// <returns>The tcpdump line as an object.</returns>
		/// <param name="line">Line.</param>
		private NtpPacket ParseComplexLine (string line)
		{
			/*
			string time = null;
			string source = null;
			string destination = null;
			string version = null;
			string type = null;
			string strat = null;
			string poll = null;
		*/
			
			return null;
		}

		/// <summary>
		/// Parses the time.
		/// </summary>
		/// <returns>The time.</returns>
		/// <param name="time">Time.</param>
		private DateTime ParseTime (string time)
		{
			string hours = time.Substring (0, 2);
			string minutes = time.Substring (3, 2);
			string seconds = time.Substring (6, 2);
			string ticks = time.Substring (9);
			
			DateTime now = DateTime.Now;
			
			int year = now.Year;
			int month = now.Month;
			int day = now.Day;
			
			DateTime approx = new DateTime (year, month, day, Int32.Parse (hours), Int32.Parse (minutes), Int32.Parse (seconds));
			DateTime exact = approx.AddTicks (UInt32.Parse (ticks) * 10);
			
			return exact;
		}

		/// <summary>
		/// Parses the end point.
		/// </summary>
		/// <returns>The end point.</returns>
		/// <param name="endPoint">End point.</param>
		private IPEndPoint ParseEndPoint (string endPoint)
		{
			string ip = endPoint.Substring (0, endPoint.LastIndexOf ('.'));
			string port = endPoint.Substring (endPoint.LastIndexOf ('.') + 1);			
			return new IPEndPoint (IPAddress.Parse (ip), Int32.Parse (port));
		}

		/// <summary>
		/// Parses the version.
		/// </summary>
		/// <returns>The version.</returns>
		/// <param name="version">Version.</param>
		private NtpMajorVersion ParseVersion (string version)
		{
			switch (version.ToLower ()) {
			case "ntpv1":
			case "v1":
				return NtpMajorVersion.NTPv1;
			case "ntpv2":
			case "v2":
				return NtpMajorVersion.NTPv2;
			case "ntpv3":
			case "v3":
				return NtpMajorVersion.NTPv3;
			case "ntpv4":
			case "v4":
				return NtpMajorVersion.NTPv4;
			default:
				return NtpMajorVersion.UnknownVersion;
			}
		}

		/// <summary>
		/// Parses the type of the packet.
		/// </summary>
		/// <returns>The packet type.</returns>
		/// <param name="type">Type.</param>
		private NtpPacketType ParsePacketType (string type)
		{
			switch (type.ToLower ()) {
			case "client":
				return NtpPacketType.Client;
			case "client alarm":
				return NtpPacketType.AlarmClient;
			case "server":
				return NtpPacketType.Server;
			case "symmetric active":
			case "sym_act":
				return NtpPacketType.ClientSymmetricActive;
			case "sym_pas":
				return NtpPacketType.SymPas;
			default:
				return NtpPacketType.UnknownPacket;
			}
		}

		NtpPacket CreatePacket (
			DateTime time,
			IPEndPoint source,
			IPEndPoint destination,
			NtpMajorVersion version,
			NtpPacketType type)
		{
			// Packet is from a host
			IEnumerable<Host> host = DataFace.Instance.Hosts.Where (h => h.Ip == source.Address.ToString ());
			if (host.Count () == 1) {
				Peer peer = DataFace.Instance.Peers.Single (p => p.Ip == destination.Address.ToString ());
				return new HostNtpPacket (time, source, destination, version, type, host.Single (), peer);
			}
			
			// Packet is from a peer
			host = DataFace.Instance.Hosts.Where (h => h.Ip == destination.Address.ToString ());
			if (host.Count () == 1) {
				SourcePeer sourcePeer;
				
				IEnumerable<SourcePeer> sourcePeers = DataFace.Instance.SourcePeers.
				Where (sp => sp.Ip.GetAddressBytes () == source.Address.GetAddressBytes ());
				
				if (sourcePeers.Count () == 0) {
					IEnumerable<Peer> peers = DataFace.Instance.Peers.Where (p => p.Ip == source.Address.ToString ());
					Peer peer = peers.Count () == 1 ? peers.Single () : null;
				
					sourcePeer = new SourcePeer (source.Address, peer);
					DataFace.Instance.SourcePeers.Save (sourcePeer);
				} else if (sourcePeers.Count () == 1) {
					sourcePeer = sourcePeers.Single ();
				} else {
					throw new ApplicationException ("Wrong number of source peer match.");
				}
				
				return new PeerNtpPacket (time, source, destination, version, type, host.Single (), sourcePeer);
			}
			
			// Packet from an unknown host.
			return null;
		}
	}
}