//
// Listener.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.net>
//
// Copyright (c) 2014 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

/*
using System;

namespace Ntp.Web.Server
{
    public class Listener
    {
        public Listener ()
        {
        }
    }
}
*/

//
// WebServer.cs
//
// Author:
//       Carsten Sonne Larsen <cs@innolan.net>
//
// Copyright (c) 2014 Carsten Sonne Larsen
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using Ntp.Analyzer.Log;

namespace Ntp.Web.Server
{
    public class WebServer : IDisposable
    {
        public WebServer (string ip, int port, int timeout, LogBase log)
        {
            IPAddress address = IPAddress.Parse (ip);
            endPoint = new IPEndPoint (address, port);
            this.timeout = timeout;
            this.log = log;
            
            servables = new Dictionary<string, IServable> ();
            serverThread = new Thread (StartServer);
        }

        private readonly IPEndPoint endPoint;
        private readonly int timeout;
        private readonly LogBase log;
        private readonly Dictionary<String, IServable> servables;
        private bool cancel;
        private Thread serverThread;

        public void AddPages (IEnumerable<IWebPage> pages)
        {
            foreach (IWebPage page in pages) {
                AddServables (page.CreatePages ());
            }
        }

        public void AddServables (IEnumerable<IServable> servables)
        {
            foreach (IServable servable in servables) {
                foreach (string locator in servable.Locators) {
                    if (this.servables.ContainsKey (locator)) {
                        log.WriteLine ("Servable location is already registered: " + locator, Severity.Debug);
                    } else {
                        this.servables.Add (locator, servable);
                    }
                }
            }      
        }

        public void Open ()
        {
            log.WriteLine ("Web server listening on " + endPoint, Severity.Notice);

            bool started = true;
       
            try {
                cancel = false;
                serverThread.Start ();
                log.WriteLine ("Started web server on thread " + serverThread.GetHashCode (), Severity.Debug);
            } catch (Exception e) {
                cancel = true;
                started = false;
                
                log.WriteLine ("Starting webserver failed.", Severity.Error);
                log.WriteLine (e);
            }
        }

        private void StartServer ()
        {
            using (Socket server = new Socket (AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)) {
                server.ReceiveTimeout = timeout;
                server.Bind (endPoint);
                server.Listen (50);
                
                while (!cancel) {
                    try {
                        using (Socket connection = server.Accept ()) {
                            if (connection.Poll (-1, SelectMode.SelectRead)) {
                                byte[] bytes = new byte[connection.Available];
                                int count = connection.Receive (bytes);
                                log.WriteLine (
                                    "Request received from " + connection.RemoteEndPoint +
                                    " at " + DateTime.Now.ToString ("dd MMM yyyy HH:mm:ss"),
                                    Severity.Info);
                                
                                connection.SendTimeout = timeout;
                                
                                string uri = ProcessHeaders (bytes);
                                IServable response = GetResponse (uri);

                                string header = response != null ?
                                    "HTTP/1.1 200 OK\r\n" +
                                                "Content-Type: " + response.ContentType + "\r\n" +
                                                "charset=utf-8\r\n" +
                                                "Connection: close\r\n\r\n"
                                    : "HTTP/1.1 404 Not found\r\n";
                                
                                connection.Send (Encoding.UTF8.GetBytes (header), header.Length, SocketFlags.None);
                                
                                if (response != null) {
                                    int i;
                                    const int size = 512;
                                    byte[] buffer = new byte[size];
                                    
                                    Stream stream = response.Generate ();
                                    stream.Position = 0;
                                    
                                    while ((i = stream.Read (buffer, 0, size)) != 0) {
                                        connection.Send (buffer, i, SocketFlags.None);
                                    }
                                }
                            }
                        }
                    } catch (Exception e) {
                        log.WriteLine (e);
                    }
                }
            }
        }

        static string ProcessHeaders (byte[] bytes)
        {
            string rawData = new string (Encoding.UTF8.GetChars (bytes)).Trim ();
            string[] lines = rawData.Split (new String[] { "\r\n", "\r", "\n" }, 100, StringSplitOptions.RemoveEmptyEntries);
                    
            // Only accept get requests
            if (lines [0].Substring (0, 3).ToUpper () != "GET")
                return null;

            string uri;            

            if (rawData.Length > 4) {
                int uriStart = rawData.IndexOf (' ') + 1;
                uri = rawData.Substring (uriStart, rawData.IndexOf (' ', uriStart) - uriStart);
            } else
                return null;
            
            Dictionary<String, String> headers = new Dictionary<String, String> ();
            
            for (int i = 1; i < lines.Length; i++) {
                int paramStart = lines [i].IndexOf (' ');
                string name = lines [i].Substring (0, paramStart - 1).Replace (":", String.Empty).ToLower ();
                string value = lines [i].Substring (paramStart + 1).ToLower ();
                
                if (!headers.ContainsKey (name))
                    headers.Add (name, value);
            }
            
            string host = headers.ContainsKey ("host") ? headers ["host"].Replace ("@", String.Empty) + "@" : String.Empty;
            
            return host + uri;
        }

        IServable GetResponse (string uri)
        {
            if (servables.ContainsKey (uri))
                return servables [uri];
            else
                return null;
        }

        private void Restart ()
        {
            Close ();
            Open ();
        }

        public void Close ()
        {
            cancel = true;
            Thread.Sleep (100);
            serverThread.Suspend ();
            log.WriteLine ("Webserver closing.", Severity.Info);
        }

        public void Dispose ()
        {
            Dispose (true);
            GC.SuppressFinalize (this);
        }

        protected virtual void Dispose (bool disposing)
        {
            if (disposing) {
                serverThread = null;
            }
        }
    }
}
